#!/usr/bin/perl

use strict;
use warnings;
use Data::Dumper;

my $file= $ARGV[0];		#input file
my $out=  $ARGV[1]; 		#output file

if($file eq '' or $out eq '')
{
    print "USAGE: consensus_exe.pl <fasta> <outfile_name>\n";
    print "Sequence in FASTA format\n";
    print "\n\tExample: ./consensus_exe.pl fasta.txt out.txt\n";
    exit();
}

open (my $fh, "<", "$file") or die "Can't open file : $!";

my $id=  "";
my $seq= "";


while(<$fh>)
{
	if(/^>(\w+)/)
	{
		$id= $1;
	}

	if(/^(\w+)/)
	{
		$seq.= $1;
	}
}
chomp $seq;

my $fasta2= "$file"."_nospaces";
open ( fasta2, ">", "$fasta2");
print fasta2 ">$id\n$seq";

#!!!!PREDICTIONS!!!!!!!


#		SVMTRIP. Available for download at http://sysbio.unl.edu/SVMTriP/.
#`cp $file ./SVMTriP/`;
system "perl SVMTriP.pl $file 20";
sleep(1);
#`(printf "$fasid\t";cat out.SVMT$fasid\_16aa_optimal_predictions.txt; echo) >> out.SVMTfull16.pos`;
#system "rm out.SVMT$fasid\_20aa_optimal_predictions.txt";
system "rm $file\_20aa_prediction.txt";


#		BCEPRED. Available for download at http://crdd.osdd.net/raghava/bcepred/index.html.
system "perl /gpsr/standalone/bcepred/bcepred.pl -i $file -o out.bcepred";
my @num;
my @aa;
my @score;

open(IN, "<", "out.bcepred");
$/="\n";
while(<IN>)
{
	if($_=~/^(\d+)\s{0,2}(\w)\s+(\S+)\s+/)
	{
		push (@num,$1);
		push (@aa,$2);
		push (@score,$3);
	}
}
$/=">";
open(OUT2, ">", "out.bcepred2");
print OUT2 "$id\n";
foreach my $i(0..$#num)
{
	if($score[$i]>2)
	{
	print OUT2 "$num[$i]\t$aa[$i]\t$score[$i]\tE\n"
	}
	else
	{
	print OUT2 "$num[$i]\t$aa[$i]\t$score[$i]\t-\n"
	}
}
print OUT2 "//\n";
system "rm out.bcepred";

sleep(1);

#		COBEPRO. Available for download upon request at http://scratch.proteomics.ics.uci.edu/. Contach cmagnan@ics.uci.edu for specifics.
`perl ./Cobepro/completescript.pl $fasta2`;
`rm $fasta2.acc`;
`rm $fasta2.ss`;
`rm $fasta2\_out.txt`;

sleep(1);


#		LBTOPE. Available for download at http://crdd.osdd.net/raghava/lbtope/protein.php.
system "perl /gpsr/standalone/lbtope/lbtope.pl -i $file -o out.lbtope";

sleep(1);

my $fasta3= "$file"."_onlyfasta";
open(OUT3, ">", $fasta3);
print OUT3 "$seq";

#		BEPIPRED. Available for download at http://www.cbs.dtu.dk/services/BepiPred-1.0/.
`python predict_bcell.py -n --noplot -m Bepipred $fasta3 > out.bepipred`;

sleep(1);


#		ABCPRED. Available for download at http://crdd.osdd.net/raghava/abcpred/index.html.
system "perl /gpsr/standalone/abcpred/proteins.pl -i $fasta2 -w 20 -o out.abcpred";

sleep(1);


#CLEANUP
`rm $fasta2`;
`rm $fasta3`;
`rm -r /gpsr/temp/*`;

#READ FILES

#OPEN

open (my $abc, "<", "out.abcpred") or die "Can't open file : $!";
open (my $bce, "<", "out.bcepred2") or die "Can't open file : $!";
open (my $bep, "<", "out.bepipred") or die "Can't open file : $!";
open (my $lbt, "<", "out.lbtope") or die "Can't open file : $!";
open (my $svm, "<", "$file"."_20aa_optimal_predictions.txt") or die "Can't open file : $!";
open (my $cob, "<", "$fasta2"."_complete_out.txt") or die "Can't open file : $!";

#OPEN ARRAYS

my @abc;
my @bce;
my @bep;
my @lbt;
my @svm;
my @cob;

#abcpred!!!
while(<$abc>)
{
    if(/^\d+\t\w+\t(\d+)\t(\S+)/)
    {

        if($2 > 0.51)
        {
        	push (@abc,$1);
        }
    }  
}
#print Dumper(@abc);
close $abc;

#bcepred!!!
$/="//";

while(<$bce>)
{

    while(/^\d+\t\w\t\S+\t(\S)/mg)
    {
		push (@bce,$1);
		#print $1;
    } 
}    
#print Dumper(@bce);
close $bce;

#bepipred!!!
while(<$bep>)
{
    while(/^\d+\t\w\t\S+\t(\S)/mg)
    {
		push (@bep,$1);
		#print $1;
    } 
}    
#print Dumper(@bep);
close $bep;

#lbtope!!!
while(<$lbt>)
{
	
    while(/^\w+\t\S+\t(\S+)\t\S+/mg)
    {
		push (@lbt,$1);
		#print $1;
    } 

}    
#print Dumper(@lbt);
close $lbt;

$/="\n";

#SVMTrip!!!
while(<$svm>)
{
    if(/^(\d+) - \d+,\s\w+,\s\S+\tE/)
    {
		push(@svm,$1);
	}
}    
#print Dumper(@svm);
close $svm;

#COBEpro!!!

while(<$cob>)
{
	
    if(/^\d/)
    {
    	chomp $_;
		@cob= split "",$_;
		#print $_;
    } 

}    
#print Dumper(@cob);
close $cob;

#SCORING

my @poscount;

my $length= length $seq;

foreach my $i(0..$length - 1)
{
	push(@poscount,0);
}
#print Dumper(@poscount);

foreach my $i(0..$length - 1)
{

	#aminoacid
	if($bce[$i] eq "E")					#bcepred
	{
		$poscount[$i]++;
	}
	
	if($cob[$i] >= 4)						#cobepro
	{
		$poscount[$i]++;
	}
	
	if($lbt[$i] eq "Epitope")				#lbtope
	{
		$poscount[$i]++;
	}
	
	if($bep[$i] eq "E")					#bepipred
	{
		$poscount[$i]++;
	}

		#models
	foreach my $j(0..$#abc)
	{
		if($abc[$j] -1 == $i)						#abcpred
		{
			for(my $k=0;$k<20;$k++)
			{
				my $sum= $i+$k;
				$poscount[$sum]++;
			}
		}
	}

	foreach my $j(0..$#abc)
	{
		if($svm[$j] - 1 == $i)						#svmtrip
		{
			for(my $k=0;$k<20;$k++)
			{
				my $sum= $i+$k;
				$poscount[$sum]++;
			}
		}
	}
}
#print Dumper(@poscount);

#CONSENSUS

my $output_string= "";

foreach my $i (0 .. $#poscount)
{
	#print $pscores[$i];
	if($poscount[$i] >= 3)
	{	
		$output_string .= "E";
	}
	else
	{	
		$output_string .= "-";
	}

}


open (OUT4, ">", "$out") or die "Can't open file : $!";
print OUT4 "$seq\n$output_string";